/* MusIRCd: an advanced Internet Relay Chat Daemon(ircd).
 * Copyright (C) 2003 by MusIRCd Development.
 *
 * $Id: m_services.c,v 1.35 2003/12/30 06:58:32 musirc Exp $
 */

#include "handlers.h"
#include "client.h"
#include "istring.h"
#include "server.h"
#include "numeric.h"
#include "send.h"
#include "modules.h"
#include "hash.h"
#include "config.h"

#define SERVICES_SERVER "services.int"
#define AIM_SERVER "aim.int"
#define NICKSERV_NICK "NickServ"
#define AIM_NICK "AIM"
#define AIMOLD_NICK "AIMSERV"
#define CHANSERV_NICK "ChanServ"
#define MEMOSERV_NICK "MemoServ"
#define HELPSERV_NICK "HelpServ"
#define STATSERV_NICK "StatServ"
#define SEENSERV_NICK "SeenServ"
#define GLOBAL_NICK "Global"
#define OPERSERV_NICK "OperServ"

static void m_nickserv(struct Client *, struct Client *, int, char **);
static void m_chanserv(struct Client *, struct Client *, int, char **);
static void m_memoserv(struct Client *, struct Client *, int, char **);
static void mo_operserv(struct Client *, struct Client *, int, char **);
static void m_helpserv(struct Client *, struct Client *, int, char **);
static void m_statserv(struct Client *, struct Client *, int, char **);
static void m_seenserv(struct Client *, struct Client *, int, char **);
static void mo_global(struct Client *, struct Client *, int, char **);
static void m_aim(struct Client *, struct Client *, int, char **);
static void m_aimold(struct Client *, struct Client *, int, char **);
static void send_service(struct Client *, const char *, const char *, int, char **);
static void send_aim(struct Client *, const char *, int, char **);

struct Message nickserv_msgtab = {
  "NICKSERV", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_nickserv, m_ignore, m_nickserv}
};

struct Message ns_msgtab = {
  "NS", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_nickserv, m_ignore, m_nickserv}
};

struct Message chanserv_msgtab = {
  "CHANSERV", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_chanserv, m_ignore, m_chanserv}
};

struct Message cs_msgtab = {
  "CS", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_chanserv, m_ignore, m_chanserv}
};

struct Message memoserv_msgtab = {
  "MEMOSERV", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_memoserv, m_ignore, m_memoserv}
};

struct Message ms_msgtab = {
  "MS", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_memoserv, m_ignore, m_memoserv}
};

struct Message operserv_msgtab = {
  "OPERSERV", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, mo_operserv}
};

struct Message os_msgtab = {
  "OS", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, mo_operserv}
};

struct Message helpserv_msgtab = {
  "HELPSERV", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_helpserv, m_ignore, m_helpserv}
};
struct Message hs_msgtab = {
  "HS", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_helpserv, m_ignore, m_helpserv}
};
struct Message statserv_msgtab = {
  "STATSERV", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_statserv, m_ignore, m_statserv}
};
struct Message seenserv_msgtab = {
  "SEENSERV", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_seenserv, m_ignore, m_seenserv}
};

struct Message global_msgtab = {
  "GLOBAL", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, mo_global}
};

struct Message aim_msgtab = {
  "AIM", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_aim, m_ignore, m_aim}
};

struct Message aimold_msgtab = {
  "AIMSERV", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_aimold, m_ignore, m_aimold}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&nickserv_msgtab);
  mod_add_cmd(&ns_msgtab);
  mod_add_cmd(&chanserv_msgtab);
  mod_add_cmd(&cs_msgtab);
  mod_add_cmd(&memoserv_msgtab);
  mod_add_cmd(&operserv_msgtab);
  mod_add_cmd(&ms_msgtab);
  mod_add_cmd(&os_msgtab);
  mod_add_cmd(&helpserv_msgtab);
  mod_add_cmd(&hs_msgtab);
  mod_add_cmd(&statserv_msgtab);
  mod_add_cmd(&seenserv_msgtab);
  mod_add_cmd(&global_msgtab);
  mod_add_cmd(&aim_msgtab);
  mod_add_cmd(&aimold_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&nickserv_msgtab);
  mod_del_cmd(&ns_msgtab);
  mod_del_cmd(&chanserv_msgtab);
  mod_del_cmd(&cs_msgtab);
  mod_del_cmd(&memoserv_msgtab);
  mod_del_cmd(&operserv_msgtab);
  mod_del_cmd(&ms_msgtab);
  mod_del_cmd(&os_msgtab);
  mod_del_cmd(&helpserv_msgtab);
  mod_del_cmd(&hs_msgtab);
  mod_del_cmd(&statserv_msgtab);
  mod_del_cmd(&seenserv_msgtab);
  mod_del_cmd(&global_msgtab);
  mod_del_cmd(&aim_msgtab);
  mod_del_cmd(&aimold_msgtab);
}

const char *_version = "$Revision: 1.35 $";
#endif

static void
send_service(struct Client *source_p, const char *nick, const char *command, int parc, char *parv[])
{
  dlink_node *next;
  dlink_node *ptr;
  struct Client *serv = find_server(SERVICES_SERVER), *target_p;

  if (!ConfigFileEntry.use_services)
  {
    sendto_one(source_p, ":%s NOTICE %s :There are no services on this network.",
               me.name, source_p->name);
    return;
  }
  if (parc < 1)
  {
    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
               me.name, source_p->name, command);
    return;
  }
  if (serv == NULL || serv == &me)
  {
    sendto_one(source_p, form_str(ERR_SERVICESDOWN), me.name, source_p->name);
    return;
  }

  DLINK_FOREACH_SAFE(ptr, next, serv->serv->users.head)
  {
    target_p = ptr->data;
    if (!irccmp(target_p->name, nick))
    {
      char buffer[BUFSIZE - 1];

      buffer[0] = '\0';
      while (1)
      {
        strlcat(buffer, parv[0], BUFSIZE - 2);
	parc--;
	parv++;
        if (!parc) break;
	strlcat(buffer, " ", BUFSIZE - 2);
      }
      if (IsCapable(serv, CAP_TS6) && HasID(target_p) && HasID(source_p))
        sendto_one(target_p, ":%s PRIVMSG %s :%s", source_p->id, target_p->id, buffer);
      else
        sendto_one(target_p, ":%s PRIVMSG %s :%s", source_p->name, target_p->name, buffer);
      return;
    }
  }
  sendto_one(source_p, form_str(ERR_SERVICESDOWN), me.name, source_p->name);
}

static void
send_aim(struct Client *source_p, const char *nick, int parc, char *parv[])
{
  dlink_node *next;
  dlink_node *ptr;
  struct Client *serv = find_server(AIM_SERVER), *target_p;

  if (serv == NULL || serv == &me)
  {
    sendto_one(source_p, form_str(ERR_SERVICESDOWN), me.name, source_p->name);
    return;
  }

  DLINK_FOREACH_SAFE(ptr, next, serv->serv->users.head)
  {
    target_p = ptr->data;
    if (!irccmp(target_p->name, nick))
    {
      char buffer[BUFSIZE - 1];

      buffer[0] = '\0';
      while (1)
      {
        strlcat(buffer, parv[0], BUFSIZE - 2);
        parc--;
        parv++;
        if (!parc) break;
        strlcat(buffer, " ", BUFSIZE - 2);
      }
      if (IsCapable(serv, CAP_TS6) && HasID(target_p) && HasID(source_p))
        sendto_one(target_p, ":%s PRIVMSG %s :%s", source_p->id, target_p->id, buffer);
      else
        sendto_one(target_p, ":%s PRIVMSG %s :%s", source_p->name, target_p->name, buffer);
      return;
    }
  }
  sendto_one(source_p, form_str(ERR_SERVICESDOWN), me.name, source_p->name);
}

static void
m_nickserv(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
{
  send_service(source_p, NICKSERV_NICK, "NICKSERV", parc - 1, parv + 1);
}

static void
m_chanserv(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
{
  send_service(source_p, CHANSERV_NICK, "CHANSERV", parc - 1, parv + 1);
}

static void
m_memoserv(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
{
  send_service(source_p, MEMOSERV_NICK, "MEMOSERV", parc - 1, parv + 1);
}

static void
mo_operserv(struct Client *client_p, struct Client *source_p,
            int parc, char *parv[])
{
  send_service(source_p, OPERSERV_NICK, "OPERSERV", parc - 1, parv + 1);
}

static void
m_helpserv(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
{
  send_service(source_p, HELPSERV_NICK, "HELPSERV", parc - 1, parv + 1);
}

static void
m_statserv(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
{
  send_service(source_p, STATSERV_NICK, "STATSERV", parc - 1, parv + 1);
}

static void
m_seenserv(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
{
  send_service(source_p, SEENSERV_NICK, "SEENSERV", parc - 1, parv + 1);
}

static void
mo_global(struct Client *client_p, struct Client *source_p,
          int parc, char *parv[])
{
  send_service(source_p, GLOBAL_NICK, "GLOBAL", parc - 1, parv + 1);
}
static void
m_aim(struct Client *client_p, struct Client *source_p,
      int parc, char *parv[])
{
  send_aim(source_p, AIM_NICK, parc - 1, parv + 1);
}
static void
m_aimold(struct Client *client_p, struct Client *source_p,
         int parc, char *parv[])
{
  send_aim(source_p, AIMOLD_NICK, parc - 1, parv + 1);
}
